/*============================================================================
  File:     AllocationBitmapDemo.sql

  Summary:  This script uses DBCC PAGE to examine
			allocation bitmaps

  Date:     June 2008

  SQL Server Versions:
		10.0.1600.22 (SS2008 RTM)
		9.00.3068.00 (SS2005 SP2+)
------------------------------------------------------------------------------
  Written By Paul S. Randal, SQLskills.com
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE MASTER;
GO

IF DATABASEPROPERTYEX ('DBMaint2008', 'Version') > 0
	DROP DATABASE DBMaint2008;
GO

CREATE DATABASE DBMaint2008;
GO

USE DBMaint2008;
GO


-- PFS page
DBCC TRACEON (3604);
GO
DBCC PAGE ('DBMaint2008', 1, 1, 3);
GO

-- GAM page
DBCC PAGE ('DBMaint2008', 1, 2, 3);
GO

-- SGAM page
DBCC PAGE ('DBMaint2008', 1, 3, 3);
GO

-- Look at the DIFF map
DBCC PAGE ('DBMaint2008', 1, 6, 3);
GO

-- Create a new table
CREATE TABLE QuickTest (
	c1 INT, c2 CHAR (8000));
INSERT INTO QuickTest VALUES (1, 'a');
GO

-- Insert rows to use up some extents
INSERT INTO QuickTest VALUES (1, 'a');
GO

-- Look at the DIFF map
DBCC PAGE ('DBMaint2008', 1, 6, 3);
GO

-- Clear it with a full backup
ALTER DATABASE DBMaint2008
	SET RECOVERY FULL;

BACKUP DATABASE DBMaint2008 TO
	DISK = 'C:\SQLskills\DBMaint.bck'
	WITH INIT;
GO
DBCC PAGE ('DBMaint2008', 1, 6, 3);
GO

-- Hmm - still some pages left CHANGED
-- What are those?

-- Force a change and show it will be
-- cleared apart from those
INSERT INTO QuickTest VALUES (1, 'a');
GO
DBCC PAGE ('DBMaint2008', 1, 6, 3);
GO

BACKUP DATABASE DBMaint2008 TO
	DISK = 'C:\SQLskills\DBMaint.bck'
	WITH INIT;
GO
DBCC PAGE ('DBMaint2008', 1, 6, 3);
GO

-- Look at the ML map
DBCC PAGE ('DBMaint2008', 1, 7, 3);
GO

-- Create an index, go into BULK_LOGGED
-- recovery mode and rebuild it
CREATE CLUSTERED INDEX QT_CL
	ON QuickTest (c1);
GO
ALTER DATABASE DBMaint2008 SET
	RECOVERY BULK_LOGGED;
GO
ALTER INDEX QT_CL ON QuickTest REBUILD;
GO
DBCC PAGE ('DBMaint2008', 1, 7, 3);
GO

-- Clear it using a log backup
ALTER DATABASE DBMaint2008
	SET RECOVERY FULL;
GO
BACKUP LOG DBMaint2008
	TO DISK = 'C:\SQLskills\DBMaint.bck'
	WITH INIT;
GO
DBCC PAGE ('DBMaint2008', 1, 7, 3);
GO

