/*============================================================================
  File:     PageRecordDemo.sql

  Summary:  This script uses DBCC IND and DBCC PAGE
			to examine on-disk structures

  Date:     June 2008

  SQL Server Versions:
		10.0.1600.22 (SS2008 RTM)
		9.00.3068.00 (SS2005 SP2+)
------------------------------------------------------------------------------
  Written By Paul S. Randal, SQLskills.com
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE MASTER;
GO

IF DATABASEPROPERTYEX ('DBMaint2008', 'Version') > 0
	DROP DATABASE DBMaint2008;
GO

CREATE DATABASE DBMaint2008;
GO

USE DBMaint2008;
GO

-- Create a test table to use
CREATE TABLE DbccPageTest (
	intCol1	INT IDENTITY,
	intCol2	INT,
	charCol	VARCHAR (8000),
	lobCol	VARCHAR (MAX));

INSERT INTO DbccPageTest VALUES (
	1, REPLICATE ('Paul', 600), REPLICATE ('a', 8000));
INSERT INTO DbccPageTest VALUES (
	2, REPLICATE ('Kimberly', 275), REPLICATE ('b', 8000));
INSERT INTO DbccPageTest VALUES (
	3, REPLICATE ('Kiera', 440), REPLICATE ('c', 8000));
INSERT INTO DbccPageTest VALUES (
	4, REPLICATE ('Katelyn', 340), REPLICATE ('d', 8000));
GO

-- Introducing DBCC IND
DBCC IND ('DBMaint2008', 'DbccPageTest', -1);
GO

-- Pick a data page and use DBCC PAGE on
-- it. We just happen to know that the
-- first page is the first listed.
-- Trace flag is required to make output
-- go to the console rather than the
-- errorlog.
-- Takes db, file, page, print option 1-3
DBCC TRACEON (3604);
GO

-- Option 1 is a hex dump of each record
-- plus interpreting the slot array.
DBCC PAGE ('DBMaint2008', 1, XX, 1);
GO

-- Data page with data records

-- Option 2 is a hex dump of the page plus
-- interpreting the slot array
DBCC PAGE ('DBMaint2008', 1, XX, 2);
GO

-- Option 3 interprets each record fully
DBCC PAGE ('DBMaint2008', 1, XX, 3);
GO

-- Note the off-row link. Let's follow it.
DBCC PAGE ('DBMaint2008', 1, XX, 3);
GO

-- Text page

-- Now let's force a forwarding record by
-- updating a row in the middle of a page.
UPDATE DbccPageTest
	SET charCol = REPLICATE ('long', 2000)
	WHERE intCol2 = 2;
GO

-- Look at the first data page again
DBCC PAGE ('DBMaint2008', 1, XX, 3);
GO

-- Now we've got a new record type. Let's
-- follow the link
DBCC PAGE ('DBMaint2008', 1, XX, 1);
GO

-- Another new record type. What's the
-- difference? The back-link...
-- Explain about NC indexes, and protection
-- against phantom reads.

-- New we create a clustered index
CREATE CLUSTERED INDEX Dbcc_CL
	ON DbccPageTest (intCol1);
GO

-- And look at the pages again, with
-- the new index ID
DBCC IND ('DBMaint2008', 'DbccPageTest', -1);
GO

-- Pick an index page - with type = 2.
DBCC PAGE ('DBMaint2008', 1, XX, 1);
GO

-- Index page with index records
-- Let's take a closer look.
DBCC PAGE ('DBMaint2008', 1, XX, 3);
GO

-- Explain the various columns and
-- UNIQUIFIERS

-- Now add a non-clustered index
CREATE NONCLUSTERED INDEX Dbcc_NCL
	ON DbccPageTest (intCol2);
GO

-- And look at the pages again, with
-- the new index ID
DBCC IND ('DBMaint2008', 'DbccPageTest', -1);
GO

-- Pick another index page and explain the
-- key columns
DBCC PAGE ('DBMaint2008', 1, XX, 3);
GO