/*============================================================================
  File:     RowStorageOrderDemo.sql

  Summary:  This script proves that records do not
			have to be physically stored in logical
			oreder on a page

			Based on: http://www.sqlskills.com/blogs/paul/2007/10/04/InsideTheStorageEngineProofThatRecordsAreNotAlwaysPhysicallyStoredInIndexKeyOrder.aspx

  Date:     June 2008

  SQL Server Versions:
		10.0.1600.22 (SS2008 RTM)
		9.00.3068.00 (SS2005 SP2+)
------------------------------------------------------------------------------
  Written By Paul S. Randal, SQLskills.com
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE MASTER;
GO

IF DATABASEPROPERTYEX ('DBMaint2008', 'Version') > 0
	DROP DATABASE DBMaint2008;
GO

CREATE DATABASE DBMaint2008;
GO

USE DBMaint2008;
GO

-- Create a test table
CREATE TABLE OrderTest (c1 INT, c2 VARCHAR (10));
CREATE CLUSTERED INDEX OrderCL ON OrderTest (c1);
GO

-- Insert values from 2 to 5, missing c1 = 1
INSERT INTO OrderTest VALUES (2, REPLICATE ('b', 10));
INSERT INTO OrderTest VALUES (3, REPLICATE ('c', 10));
INSERT INTO OrderTest VALUES (4, REPLICATE ('d', 10));
INSERT INTO OrderTest VALUES (5, REPLICATE ('e', 10));
GO

-- Look at the page
EXEC sp_AllocationMetadata 'OrderTest';
GO

DBCC TRACEON (3604);
GO

DBCC PAGE ('DBMaint2008', 1, XX, 3);
GO

-- Now insert c1 = 1 and look at the page again
INSERT INTO OrderTest VALUES (1, REPLICATE ('a', 10));
GO

DBCC PAGE ('DBMaint2008', 1, XX, 3);
GO

-- Check out the record *offset* on the page
-- even though it's slot # 0

-- Now do a dump-style 2
DBCC PAGE ('DBMaint2008', 1, XX, 2);
GO

-- And a dump-style 1 to get the slot array
-- just to hammer it home
DBCC PAGE ('DBMaint2008', 1, XX, 1);
GO

